@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_blit_N.c,v 1.3 2001/07/07 20:20:17 hercules Exp $";
+ "@(#) $Id: SDL_blit_N.c,v 1.7 2002/08/01 23:06:39 slouken Exp $";
 #endif
 
 #include <stdio.h>
@@ -49,7 +49,6 @@ static char rcsid =
 #define HermesConverterInterface	SDL_BlitInfo
 #define HermesClearInterface		void
 #define STACKCALL
-typedef Uint32 int32;
 
 #include "HeadMMX.h"
 #include "HeadX86.h"
@@ -1203,7 +1202,7 @@ static void BlitNtoN(SDL_BlitInfo *info)
 	int srcbpp = srcfmt->BytesPerPixel;
 	SDL_PixelFormat *dstfmt = info->dst;
 	int dstbpp = dstfmt->BytesPerPixel;
-	unsigned alpha = dstfmt->Amask ? SDL_ALPHA_OPAQUE : 0;
+	unsigned alpha = dstfmt->Amask ? srcfmt->alpha : 0;
 
 	while ( height-- ) {
 		DUFFS_LOOP(
@@ -1359,7 +1358,7 @@ static void BlitNtoNKey(SDL_BlitInfo *info)
 	SDL_PixelFormat *dstfmt = info->dst;
 	int srcbpp = srcfmt->BytesPerPixel;
 	int dstbpp = dstfmt->BytesPerPixel;
-	unsigned alpha = dstfmt->Amask ? SDL_ALPHA_OPAQUE : 0;
+	unsigned alpha = dstfmt->Amask ? srcfmt->alpha : 0;
 
 	while ( height-- ) {
 		DUFFS_LOOP(
