@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_DirectFB_video.h,v 1.4 2002/03/06 11:23:03 slouken Exp $";
+ "@(#) $Id: SDL_DirectFB_video.h,v 1.6 2002/08/31 04:13:28 slouken Exp $";
 #endif
 
 #ifndef _SDL_DirectFB_video_h
@@ -45,11 +45,12 @@ struct SDL_PrivateVideoData
   IDirectFBDisplayLayer *layer;
   IDirectFBEventBuffer  *eventbuffer;
 
-#define NUM_MODELISTS	4		/* 8, 16, 24, and 32 bits-per-pixel */
-  int SDL_nummodes[NUM_MODELISTS];
-  SDL_Rect **SDL_modelist[NUM_MODELISTS];
+  int nummodes;
+  SDL_Rect **modelist;
 };
 
 #define HIDDEN (this->hidden)
 
+void SetDirectFBerror (const char *function, DFBResult code);
+
 #endif /* _SDL_DirectFB_video_h */
