@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_ph_modes_c.h,v 1.2 2001/04/26 16:50:19 hercules Exp $";
+ "@(#) $Id: SDL_ph_modes_c.h,v 1.6 2002/03/23 20:19:44 slouken Exp $";
 #endif
 
 
@@ -31,7 +31,8 @@ static char rcsid =
 
 #include "SDL_ph_video.h"
 
-extern int ph_GetVideoModes(_THIS);
+#define PH_MAX_VIDEOMODES 127
+
 extern SDL_Rect **ph_ListModes(_THIS,SDL_PixelFormat *format, Uint32 flags);
 extern void ph_FreeVideoModes(_THIS);
 extern int ph_ResizeFullScreen(_THIS);
