@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_sysevents.c,v 1.9 2001/09/13 16:15:43 slouken Exp $";
+ "@(#) $Id: SDL_sysevents.c,v 1.22 2002/08/20 05:59:31 slouken Exp $";
 #endif
 
 #include <stdlib.h>
@@ -47,16 +47,25 @@ static char rcsid =
 
 #ifdef _WIN32_WCE
 #define NO_GETKEYBOARDSTATE
+#define NO_CHANGEDISPLAYSETTINGS
 #endif
 
 /* The window we use for everything... */
-const char *SDL_Appname = NULL;
+#ifdef _WIN32_WCE
+LPWSTR SDL_Appname = NULL;
+#else
+LPSTR SDL_Appname = NULL;
+#endif
 HINSTANCE SDL_Instance = NULL;
 HWND SDL_Window = NULL;
 RECT SDL_bounds = {0, 0, 0, 0};
 int SDL_resizing = 0;
 int mouse_relative = 0;
 int posted = 0;
+#ifndef NO_CHANGEDISPLAYSETTINGS
+DEVMODE SDL_fullscreen_mode;
+#endif
+WORD *gamma_saved = NULL;
 
 
 /* Functions called by the message processing function */
@@ -64,8 +73,23 @@ LONG
 (*HandleMessage)(_THIS, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)=NULL;
 void (*WIN_RealizePalette)(_THIS);
 void (*WIN_PaletteChanged)(_THIS, HWND window);
-void (*WIN_SwapGamma)(_THIS);
 void (*WIN_WinPAINT)(_THIS, HDC hdc);
+extern void DIB_SwapGamma(_THIS);
+
+static void SDL_RestoreGameMode(void)
+{
+#ifndef NO_CHANGEDISPLAYSETTINGS
+	ShowWindow(SDL_Window, SW_RESTORE);
+	ChangeDisplaySettings(&SDL_fullscreen_mode, CDS_FULLSCREEN);
+#endif
+}
+static void SDL_RestoreDesktopMode(void)
+{
+#ifndef NO_CHANGEDISPLAYSETTINGS
+	ShowWindow(SDL_Window, SW_MINIMIZE);
+	ChangeDisplaySettings(NULL, 0);
+#endif
+}
 
 #ifdef WM_MOUSELEAVE
 /* 
@@ -113,6 +137,7 @@ static void WIN_GetKeyboardState(void)
 #ifndef NO_GETKEYBOARDSTATE
 	SDLMod state;
 	BYTE keyboard[256];
+	Uint8 *kstate = SDL_GetKeyState(NULL);
 
 	state = KMOD_NONE;
 	if ( GetKeyboardState(keyboard) ) {
@@ -136,9 +161,11 @@ static void WIN_GetKeyboardState(void)
 		}
 		if ( keyboard[VK_NUMLOCK] & 0x01) {
 			state |= KMOD_NUM;
+			kstate[SDLK_NUMLOCK] = SDL_PRESSED;
 		}
 		if ( keyboard[VK_CAPITAL] & 0x01) {
 			state |= KMOD_CAPS;
+			kstate[SDLK_CAPSLOCK] = SDL_PRESSED;
 		}
 	}
 	SDL_SetModState(state);
@@ -177,7 +204,12 @@ LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 					WIN_GrabInput(this, SDL_GRAB_ON);
 				}
 				if ( !(SDL_GetAppState()&SDL_APPINPUTFOCUS) ) {
-					WIN_SwapGamma(this);
+					if ( ! DDRAW_FULLSCREEN() ) {
+						DIB_SwapGamma(this);
+					}
+					if ( WINDIB_FULLSCREEN() ) {
+						SDL_RestoreGameMode();
+					}
 				}
 				posted = SDL_PrivateAppActive(1, appstate);
 				WIN_GetKeyboardState();
@@ -191,7 +223,12 @@ LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 					WIN_GrabInput(this, SDL_GRAB_OFF);
 				}
 				if ( SDL_GetAppState() & SDL_APPINPUTFOCUS ) {
-					WIN_SwapGamma(this);
+					if ( ! DDRAW_FULLSCREEN() ) {
+						DIB_SwapGamma(this);
+					}
+					if ( WINDIB_FULLSCREEN() ) {
+						SDL_RestoreDesktopMode();
+					}
 				}
 				posted = SDL_PrivateAppActive(0, appstate);
 			}
@@ -277,27 +314,27 @@ LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 				/* Figure out which button to use */
 				switch (msg) {
 					case WM_LBUTTONDOWN:
-						button = 1;
+						button = SDL_BUTTON_LEFT;
 						state = SDL_PRESSED;
 						break;
 					case WM_LBUTTONUP:
-						button = 1;
+						button = SDL_BUTTON_LEFT;
 						state = SDL_RELEASED;
 						break;
 					case WM_MBUTTONDOWN:
-						button = 2;
+						button = SDL_BUTTON_MIDDLE;
 						state = SDL_PRESSED;
 						break;
 					case WM_MBUTTONUP:
-						button = 2;
+						button = SDL_BUTTON_MIDDLE;
 						state = SDL_RELEASED;
 						break;
 					case WM_RBUTTONDOWN:
-						button = 3;
+						button = SDL_BUTTON_RIGHT;
 						state = SDL_PRESSED;
 						break;
 					case WM_RBUTTONUP:
-						button = 3;
+						button = SDL_BUTTON_RIGHT;
 						state = SDL_RELEASED;
 						break;
 					default:
@@ -338,11 +375,13 @@ LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 				if ( move ) {
 					Uint8 button;
 					if ( move > 0 )
-						button = 4;
+						button = SDL_BUTTON_WHEELUP;
 					else
-						button = 5;
+						button = SDL_BUTTON_WHEELDOWN;
 					posted = SDL_PrivateMouseButton(
 						SDL_PRESSED, button, 0, 0);
+					posted |= SDL_PrivateMouseButton(
+						SDL_RELEASED, button, 0, 0);
 				}
 			}
 			return(0);
@@ -404,24 +443,22 @@ LONG CALLBACK WinMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 		return(0);
 #endif /* WM_GETMINMAXINFO */
 
-		case WM_MOVE: {
+		case WM_WINDOWPOSCHANGED: {
 			SDL_VideoDevice *this = current_video;
+			int w, h;
 
 			GetClientRect(SDL_Window, &SDL_bounds);
 			ClientToScreen(SDL_Window, (LPPOINT)&SDL_bounds);
 			ClientToScreen(SDL_Window, (LPPOINT)&SDL_bounds+1);
+			w = SDL_bounds.right-SDL_bounds.left;
+			h = SDL_bounds.bottom-SDL_bounds.top;
 			if ( this->input_grab != SDL_GRAB_OFF ) {
 				ClipCursor(&SDL_bounds);
 			}
-		}
-		break;
-	
-		case WM_SIZE: {
-			if ( SDL_PublicSurface &&
+			if ( SDL_PublicSurface && 
 				(SDL_PublicSurface->flags & SDL_RESIZABLE) ) {
-				SDL_PrivateResize(LOWORD(lParam), HIWORD(lParam));
+				SDL_PrivateResize(w, h);
 			}
-			return(0);
 		}
 		break;
 
@@ -544,22 +581,23 @@ int SDL_RegisterApp(char *name, Uint32 style, void *hInst)
 	/* Register the application class */
 	class.hCursor		= NULL;
 #ifdef _WIN32_WCE
-    {
-	/* WinCE uses the UNICODE version */
-	int nLen = strlen(name)+1;
-	LPWSTR lpszW = alloca(nLen*2);
-	MultiByteToWideChar(CP_ACP, 0, name, -1, lpszW, nLen);
-	class.hIcon		= LoadImage(hInst, lpszW, IMAGE_ICON,
+	{
+		/* WinCE uses the UNICODE version */
+		int nLen = strlen(name)+1;
+		SDL_Appname = malloc(nLen*2);
+		MultiByteToWideChar(CP_ACP, 0, name, -1, SDL_Appname, nLen);
+	}
+#else
+	{
+		int nLen = strlen(name)+1;
+		SDL_Appname = malloc(nLen);
+		strcpy(SDL_Appname, name);
+	}
+#endif /* _WIN32_WCE */
+	class.hIcon		= LoadImage(hInst, SDL_Appname, IMAGE_ICON,
 	                                    0, 0, LR_DEFAULTCOLOR);
 	class.lpszMenuName	= NULL;
-	class.lpszClassName	= lpszW;
-    }
-#else
-	class.hIcon		= LoadImage(hInst, name, IMAGE_ICON,
-	                                    0, 0, LR_DEFAULTCOLOR);
-	class.lpszMenuName	= "(none)";
-	class.lpszClassName	= name;
-#endif /* _WIN32_WCE */
+	class.lpszClassName	= SDL_Appname;
 	class.hbrBackground	= NULL;
 	class.hInstance		= hInst;
 	class.style		= style;
@@ -573,7 +611,6 @@ int SDL_RegisterApp(char *name, Uint32 style, void *hInst)
 		SDL_SetError("Couldn't register application class");
 		return(-1);
 	}
-	SDL_Appname = name;
 	SDL_Instance = hInst;
 
 #ifdef WM_MOUSELEAVE
