@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_x11video.c,v 1.5 2001/11/07 17:59:07 slouken Exp $";
+ "@(#) $Id: SDL_x11video.c,v 1.13 2002/09/30 00:35:25 slouken Exp $";
 #endif
 
 /* X11 based SDL video driver implementation.
@@ -282,7 +282,8 @@ static void create_aux_windows(_THIS)
     xattr.border_pixel = 0;
     xattr.colormap = SDL_XColorMap;
 
-    FSwindow = XCreateWindow(SDL_Display, SDL_Root, 0, 0, 32, 32, 0,
+    FSwindow = XCreateWindow(SDL_Display, SDL_Root,
+                             xinerama_x, xinerama_y, 32, 32, 0,
 			     this->hidden->depth, InputOutput, SDL_Visual,
 			     CWOverrideRedirect | CWBackPixel | CWBorderPixel
 			     | CWColormap,
@@ -350,8 +351,12 @@ static void create_aux_windows(_THIS)
 	XClassHint *classhints;
 	classhints = XAllocClassHint();
 	if(classhints != NULL) {
-	    classhints->res_name = "SDL_App";
-	    classhints->res_class = "SDL_App";
+            char *classname = getenv("SDL_VIDEO_X11_WMCLASS");
+            if ( ! classname ) {
+                classname = "SDL_App";
+            }
+	    classhints->res_name = classname;
+	    classhints->res_class = classname;
 	    XSetClassHint(SDL_Display, WMwindow, classhints);
 	    XFree(classhints);
 	}
@@ -404,26 +409,14 @@ static int X11_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	/* use default screen (from $DISPLAY) */
 	SDL_Screen = DefaultScreen(SDL_Display);
 
+	use_mitshm = 0;
 #ifndef NO_SHARED_MEMORY
 	/* Check for MIT shared memory extension */
-	use_mitshm = 0;
 	if ( local_X11 ) {
 		use_mitshm = XShmQueryExtension(SDL_Display);
 	}
 #endif /* NO_SHARED_MEMORY */
 
-	/* See whether or not we need to swap pixels */
-	swap_pixels = 0;
-	if ( SDL_BYTEORDER == SDL_LIL_ENDIAN ) {
-		if ( XImageByteOrder(SDL_Display) == MSBFirst ) {
-			swap_pixels = 1;
-		}
-	} else {
-		if ( XImageByteOrder(SDL_Display) == LSBFirst ) {
-			swap_pixels = 1;
-		}
-	}
-
 	/* Get the available video modes */
 	if(X11_GetVideoModes(this) < 0)
 	    return -1;
@@ -521,6 +514,26 @@ static void X11_DestroyWindow(_THIS, SDL_Surface *screen)
 	}
 }
 
+static SDL_bool X11_WindowPosition(_THIS, int *x, int *y, int w, int h)
+{
+	const char *window = getenv("SDL_VIDEO_WINDOW_POS");
+	const char *center = getenv("SDL_VIDEO_CENTERED");
+	if ( window ) {
+		if ( sscanf(window, "%d,%d", x, y) == 2 ) {
+			return SDL_TRUE;
+		}
+		if ( strcmp(window, "center") == 0 ) {
+			center = window;
+		}
+	}
+	if ( center ) {
+		*x = (DisplayWidth(SDL_Display, SDL_Screen) - w)/2;
+		*y = (DisplayHeight(SDL_Display, SDL_Screen) - h)/2;
+		return SDL_TRUE;
+	}
+	return SDL_FALSE;
+}
+
 static void X11_SetSizeHints(_THIS, int w, int h, Uint32 flags)
 {
 	XSizeHints *hints;
@@ -543,13 +556,7 @@ static void X11_SetSizeHints(_THIS, int w, int h, Uint32 flags)
 			hints->flags |= USPosition;
 		} else
 		/* Center it, if desired */
-		if ( getenv("SDL_VIDEO_CENTERED") ) {
-			int display_w, display_h;
-
-			display_w = DisplayWidth(SDL_Display, SDL_Screen);
-			display_h = DisplayHeight(SDL_Display, SDL_Screen);
-			hints->x = (display_w - w)/2;
-			hints->y = (display_h - h)/2;
+		if ( X11_WindowPosition(this, &hints->x, &hints->y, w, h) ) {
 			hints->flags |= USPosition;
 			XMoveWindow(SDL_Display, WMwindow, hints->x, hints->y);
 
@@ -845,6 +852,30 @@ static int X11_CreateWindow(_THIS, SDL_Surface *screen,
 	/* Update the internal keyboard state */
 	X11_SetKeyboardState(SDL_Display, NULL);
 
+	/* When the window is first mapped, ignore non-modifier keys */
+	{
+		Uint8 *keys = SDL_GetKeyState(NULL);
+		for ( i = 0; i < SDLK_LAST; ++i ) {
+			switch (i) {
+			    case SDLK_NUMLOCK:
+			    case SDLK_CAPSLOCK:
+			    case SDLK_LCTRL:
+			    case SDLK_RCTRL:
+			    case SDLK_LSHIFT:
+			    case SDLK_RSHIFT:
+			    case SDLK_LALT:
+			    case SDLK_RALT:
+			    case SDLK_LMETA:
+			    case SDLK_RMETA:
+			    case SDLK_MODE:
+				break;
+			    default:
+				keys[i] = SDL_RELEASED;
+				break;
+			}
+		}
+	}
+
 	/* Map them both and go fullscreen, if requested */
 	if ( ! SDL_windowid ) {
 		XMapWindow(SDL_Display, SDL_Window);
